@@ -1,20 +1,12 @@
-# -*- coding: utf-8 -*-
-
 """Helper functions for working with version control systems."""
-
-from __future__ import unicode_literals
 import logging
 import os
-import subprocess
-
-try:
-    from shutil import which
-except ImportError:
-    from whichcraft import which
+import subprocess  # nosec
+from shutil import which
 
 from cookiecutter.exceptions import (
-    RepositoryNotFound,
     RepositoryCloneFailed,
+    RepositoryNotFound,
     UnknownRepoType,
     VCSNotInstalled,
 )
@@ -70,6 +62,7 @@ def clone(repo_url, checkout=None, clone_to_dir='.', no_input=False):
     :param clone_to_dir: The directory to clone to.
                          Defaults to the current directory.
     :param no_input: Suppress all user prompts when calling via API.
+    :returns: str with path to the new directory of the repository.
     """
     # Ensure that clone_to_dir exists
     clone_to_dir = os.path.expanduser(clone_to_dir)
@@ -80,16 +73,17 @@ def clone(repo_url, checkout=None, clone_to_dir='.', no_input=False):
 
     # check that the appropriate VCS for the repo_type is installed
     if not is_vcs_installed(repo_type):
-        msg = "'{0}' is not installed.".format(repo_type)
+        msg = f"'{repo_type}' is not installed."
         raise VCSNotInstalled(msg)
 
     repo_url = repo_url.rstrip('/')
-    tail = os.path.split(repo_url)[1]
+    repo_name = os.path.split(repo_url)[1]
     if repo_type == 'git':
-        repo_dir = os.path.normpath(os.path.join(clone_to_dir, tail.rsplit('.git')[0]))
-    elif repo_type == 'hg':
-        repo_dir = os.path.normpath(os.path.join(clone_to_dir, tail))
-    logger.debug('repo_dir is %s', repo_dir)
+        repo_name = repo_name.split(':')[-1].rsplit('.git')[0]
+        repo_dir = os.path.normpath(os.path.join(clone_to_dir, repo_name))
+    if repo_type == 'hg':
+        repo_dir = os.path.normpath(os.path.join(clone_to_dir, repo_name))
+    logger.debug(f'repo_dir is {repo_dir}')
 
     if os.path.isdir(repo_dir):
         clone = prompt_and_delete(repo_dir, no_input=no_input)
@@ -98,13 +92,13 @@ def clone(repo_url, checkout=None, clone_to_dir='.', no_input=False):
 
     if clone:
         try:
-            subprocess.check_output(
+            subprocess.check_output(  # nosec
                 [repo_type, 'clone', repo_url],
                 cwd=clone_to_dir,
                 stderr=subprocess.STDOUT,
             )
             if checkout is not None:
-                subprocess.check_output(
+                subprocess.check_output(  # nosec
                     [repo_type, 'checkout', checkout],
                     cwd=repo_dir,
                     stderr=subprocess.STDOUT,
@@ -121,6 +115,7 @@ def clone(repo_url, checkout=None, clone_to_dir='.', no_input=False):
                     'The {} branch of repository {} could not found, '
                     'have you made a typo?'.format(checkout, repo_url)
                 )
+            logger.error('git clone failed with error: %s', output)
             raise
 
     return repo_dir
