@@ -1,7 +1,4 @@
-# -*- coding: utf-8 -*-
-
 """Collection of tests around loading extensions."""
-
 import pytest
 
 from cookiecutter.environment import StrictEnvironment
@@ -25,3 +22,4 @@ def test_env_should_come_with_default_extensions():
     assert 'cookiecutter.extensions.JsonifyExtension' in env.extensions
     assert 'cookiecutter.extensions.RandomStringExtension' in env.extensions
     assert 'cookiecutter.extensions.SlugifyExtension' in env.extensions
+    assert 'cookiecutter.extensions.UUIDExtension' in env.extensions
